import shutil
import time

import pyautogui
import datetime

import browser_utils as br
from config_module import config
from image_utils import _take_screenshot
from internal import _wait_for_element
import internal
from ocr_utils import _perform_ocr
from realswift.exceptions import InvalidScrollOptionException, ElementNotFoundException
from reporter import report

retries = config.retries
screenshots_folder = config.screenshots_folder
screenshots_path = config.screenshots_path
output_path = config.output_path
element_ss_path = config.element_ss_path
offsetx_default = config.offsetx_default
offsety_default = config.offsety_default

mousespeed = config.mousespeed
found = False
flag_executed = False
def recapture():
    """
        Capture a screenshot and perform Optical Character Recognition (OCR) on it.
    """
    _take_screenshot()
    _perform_ocr()
def press_keys(keyToPress,noOfPresses=1):
    """
           Simulate pressing a keyborad button.
           available values check here : https://pyautogui.readthedocs.io/en/latest/keyboard.html#keyboard-keys

           Args:
               keyToPress (str): The keyboard button to press.
    """
    br._activate_window
    pyautogui.press(keyToPress,noOfPresses)
def type_keys(txtToEnter):
    """
           Type text.
           available values check here : https://pyautogui.readthedocs.io/en/latest/keyboard.html#keyboard-keys

           Args:
               txtToEnter (str): The text to type.
    """
    br._activate_window
    pyautogui.typewrite(txtToEnter)
    print(f"typed '{txtToEnter}'")
def type( txtToEnter, txtToFind, exactmatch=True, item_position=1, relative_to_word_in_x="",
         relative_to_word_in_y="",offsetx=offsetx_default,offsety=offsety_default):
    """
        Simulate typing a string and clicking on a specified element.

        Args:
            txtToEnter (str): The text to type.
            txtToFind (str): The element to find and type.
            exactmatch (bool): Whether to perform an exact match for element.
            item_position (int): The index of the item_position to interact with.
            relative_to_word_in_x (str): A reference word in x-axis direction to uniquely identify the element.
            relative_to_word_in_y (str): A reference word in y-axis direction to uniquely identify the element.
            offsetx (int): Offset for the identified x-coordinate
            offsety (int): Offset for the identified y-coordinate

    """
    start_time = datetime.datetime.now()

    if not txtToFind == "":
        midpoint = _wait_for_element(txtToFind, exactmatch, "", item_position, relative_to_word_in_x, relative_to_word_in_y)
        x = midpoint[0] - offsetx
        y = midpoint[1] - offsety
        br._activate_window
        pyautogui.moveTo(x, y, mousespeed)
    br._activate_window
    pyautogui.click()
    br._activate_window
    pyautogui.typewrite(txtToEnter)
    print(f"typed '{txtToEnter}'")

    time_diff = datetime.datetime.now() - start_time
    time_diff = str(time_diff)[:-4]
    report(f"Found element '{txtToFind}' and typed '{txtToEnter}'","Passed",time_diff,new_file_path)

def click(txtToFind, exactmatch=True, item_position=1, relative_word_in_x_direction="", relative_word_in_y_direction="",offsetx=offsetx_default,offsety=offsety_default):
    """
    Click on a specified element.

    Args
        txtToFind (str): The element to find and click on.
        exactmatch (bool): Whether to perform an exact match for the element.
        object (str): The element object to interact with.
        item_position (int): The index of the item_position to interact with.
        relative_word_in_x_direction (str): A reference word in x-axis direction to uniquely identify the element.
        relative_word_in_y_direction (str): A reference word in y-axis direction to uniquely identify the element.
        offsetx (int): Offset for the identified x-coordinate
        offsety (int): Offset for the identified y-coordinate
    """
    start_time = datetime.datetime.now()

    midpoint = _wait_for_element(txtToFind, exactmatch,"", item_position, relative_word_in_x_direction, relative_word_in_y_direction)
    x = midpoint[0] - offsetx
    y = midpoint[1] - offsety
    br._activate_window
    pyautogui.moveTo(x, y, mousespeed)
    pyautogui.click()
    print(f"clicked on the element '{txtToFind}'")

    time_diff = datetime.datetime.now() - start_time
    time_diff = str(time_diff)[:-4]
    report(f"Clicked on the element '{txtToFind}'","Passed",time_diff,internal.new_file_path)

def click_img_object(object, item_position=1, relative_word_in_x_direction="", relative_word_in_y_direction="",offsetx=offsetx_default,offsety=offsety_default):
    """
    Click on a specified image object.(Take screenshot and place in img_objects folder ex:-submit.png)

    Args
        object (str): The element object to interact with.
        item_position (int): The index of the item_position to interact with.
        relative_word_in_x_direction (str): A reference word in x-axis direction to uniquely identify the element.
        relative_word_in_y_direction (str): A reference word in y-axis direction to uniquely identify the element.
        offsetx (int): Offset for the identified x-coordinate
        offsety (int): Offset for the identified y-coordinate
    """
    start_time = datetime.datetime.now()

    midpoint = _wait_for_element("", True, object, item_position, relative_word_in_x_direction, relative_word_in_y_direction)
    x = midpoint[0] - offsetx
    y = midpoint[1] - offsety
    br._activate_window
    pyautogui.moveTo(x, y, mousespeed)
    pyautogui.click()
    print(f"clicked on the img object '{object}'")

    time_diff = datetime.datetime.now() - start_time
    time_diff = str(time_diff)[:-4]
    report(f"Clicked on the img object '{object}''","Passed",time_diff,internal.new_file_path)


def scroll_find_click(txtToFind,direction="down", scrolls=1,exactmatch=True,offsetx=offsetx_default,offsety=offsety_default):
    """
        Scroll the screen in a specified direction and find text and click

        Args:
            txtToFind (str): The element to find during scrolling.
            direction (str): The direction of scrolling ("up", "down", "left", "right").
            scrolls (int): The number of times to scroll.
            exactmatch (bool): Whether to perform an exact match for the element.
            offsetx (int): Offset for the identified x-coordinate
            offsety (int): Offset for the identified y-coordinate
        """
    start_time = datetime.datetime.now()
    midpoint = scroll_find(txtToFind,direction,scrolls,exactmatch)
    x = midpoint[0] - offsetx
    y = midpoint[1] - offsety
    br._activate_window
    pyautogui.moveTo(x, y, mousespeed)
    pyautogui.click()
    time_diff = datetime.datetime.now() - start_time
    time_diff = str(time_diff)[:-4]
    print(f"scrolled and clicked on the element '{txtToFind}'")
    report(f"Scrolled and clicked on the element '{txtToFind}'","Passed",time_diff,internal.new_file_path)

def scroll_find( txtToFind,direction="down", scrolls=1,exactmatch=True):
    """
    Scroll the screen in a specified direction and find text.

    Args:
        txtToFind (str): The element to find during scrolling.
        direction (str): The direction of scrolling ("up", "down", "left", "right").
        scrolls (int): The number of times to scroll.
        exactmatch (bool): Whether to perform an exact match for the element.
    """
    global results
    midpoint = False
    found = False
    start_try = 0
    retries = scrolls
    while not found and start_try < retries:
        if not txtToFind == "":
            _take_screenshot()
            _perform_ocr()
            midpoint = _wait_for_element(txtToFind, exactmatch, "", 0, "", "", True)
        if txtToFind != "":
            if not midpoint:
                print(f"element '{txtToFind}' not found after scrolling in {direction} direction...scrolling further")
            else:
                #print(f"element '{txtToFind}' found after scrolling in {direction} direction")
                found = True
                break
        scroll(direction,scrolls)
        start_try = start_try + 1
    if not found:
        try:
            raise ElementNotFoundException(f"element '{txtToFind}' not found within {retries} scrolls...exiting the script")
        except ElementNotFoundException as ce:
            print(f"Caught an exception: {ce.message}")
            report(f"Element '{txtToFind}' not found within {retries} scrolls", "Failed", "NA", screenshots_path)
            br.tear_down()
            exit(1)
        exit(1)
    results = []
    return midpoint

def scroll(direction="down",scrolls=1):
    scroll_duration = 0.1
    if direction == "up":
        print(f"scrolling {direction}...")
        for _ in range(scrolls):
            pyautogui.press('up')
            time.sleep(scroll_duration)
    elif direction == "down":
        print(f"scrolling {direction}...")
        for _ in range(scrolls):
            pyautogui.press('down')
            time.sleep(scroll_duration)
    elif direction == "left":
        print(f"scrolling {direction}...")
        for _ in range(scrolls):
            pyautogui.press('left')
            time.sleep(scroll_duration)
    elif direction == "right":
        print(f"scrolling {direction}...")
        for _ in range(scrolls):
            pyautogui.press('right')
            time.sleep(scroll_duration)
    else:
        try:
            raise InvalidScrollOptionException("not a valid scroll input")
        except InvalidScrollOptionException as ce:
            print(f"Caught an exception: {ce.message}")
            exit(1)


def hover(txtToFind, exactmatch=True, item_position=1, relative_word_in_x_direction="", relative_word_in_y_direction="",offsetx=offsetx_default,offsety=offsety_default):
    """
        Hover over a specified element.

        Args:
            txtToFind (str): The element to find and hover over.
            exactmatch (bool): Whether to perform an exact match for the element.
            item_position (int): The index of the item_position to interact with.
            relative_to_word_in_x (str): A reference word in x-axis direction to uniquely identify the element.
            relative_to_word_in_y (str): A reference word in y-axis direction to uniquely identify the element.
            hover (bool): Whether to hover over the element.
            offsetx (int): Offset for the identified x-coordinate
            offsety (int): Offset for the identified y-coordinate
    """
    start_time = datetime.datetime.now()

    midpoint = _wait_for_element(txtToFind, exactmatch, "", item_position, relative_word_in_x_direction,
                                 relative_word_in_y_direction)
    x = midpoint[0] - offsetx
    y = midpoint[1] - offsety
    br._activate_window
    pyautogui.moveTo(x, y, mousespeed)
    print(f"hovered on the element '{txtToFind}'")

    time_diff = datetime.datetime.now() - start_time
    time_diff = str(time_diff)[:-4]
    report(f"Hovered on the element '{txtToFind}'","Passed",time_diff,internal.new_file_path)

def hover_img_object(object, item_position=1, relative_word_in_x_direction="", relative_word_in_y_direction="",offsetx=offsetx_default,offsety=offsety_default):
    """
    Hover on a specified image object.(Take screenshot and place in img_objects folder ex:-submit.png)

    Args
        object (str): The image object to interact with.
        item_position (int): The index of the item_position to interact with.
        relative_word_in_x_direction (str): A reference word in x-axis direction to uniquely identify the element.
        relative_word_in_y_direction (str): A reference word in y-axis direction to uniquely identify the element.
        offsetx (int): Offset for the identified x-coordinate
        offsety (int): Offset for the identified y-coordinate
    """
    start_time = datetime.datetime.now()

    midpoint = _wait_for_element("", True, object, item_position, relative_word_in_x_direction, relative_word_in_y_direction)
    x = midpoint[0] - offsetx
    y = midpoint[1] - offsety
    br._activate_window
    pyautogui.moveTo(x, y, mousespeed)
    pyautogui.click()
    print(f"clicked on the img object '{object}'")

    time_diff = datetime.datetime.now() - start_time
    time_diff = str(time_diff)[:-4]
    report(f"Hovered on the img object '{object}''","Passed",time_diff,internal.new_file_path)



import os
import subprocess
import sys
import time
import shutil
from config_module import config

browser_path = config.browser_path
browser = config.browser
screenshots_folder = config.screenshots_folder
options = config.options.split(";")
window_chk = any("--window-size" in element for element in options)
if window_chk:
    options.append("--window-position=0,0")


def open_browser(url):
    global proc
    # try:
    if os.path.isdir(screenshots_folder):
        shutil.rmtree(screenshots_folder)
    os.mkdir(screenshots_folder)
    os.mkdir(screenshots_folder + "/old_" + screenshots_folder)
    # except Exception as e:
    #    print(f"{e}")

    proc = subprocess.Popen([browser_path, url] + options, stdout=subprocess.DEVNULL,
                            stderr=subprocess.DEVNULL,
                            close_fds=True)
    time.sleep(2)


def tear_down():
    proc.kill
    exit(0)


def _activate_window():
    if sys.platform == "win32":
        import pygetwindow as gw
        try:
            if browser == "chrome":
                active_window = gw.getWindowsWithTitle("Google Chrome")
                active_window[0].activate()
            elif browser == "edge":
                active_window = gw.getWindowsWithTitle("Edge")
                active_window[0].activate()
            elif browser == "firefox":
                active_window = gw.getWindowsWithTitle("Firefox")
                active_window[0].activate()
        except Exception as e:
            print(f"window activation failed with error {e}")
        return active_window[0]



import sys
import time
import datetime
import shutil

from config_module import config
from image_utils import _find_image,_take_screenshot
from ocr_utils import _perform_ocr,_analyse_ocr_results
from exceptions import ElementNotFoundException
from realswift.browser_utils import tear_down
from reporter import report
browser = config.browser
retries = config.retries
screenshots_folder = config.screenshots_folder
screenshots_path = config.screenshots_path
output_path = config.output_path
element_ss_path = config.element_ss_path
mousespeed = config.mousespeed
found = False
flag_executed = False

def _wait_for_element(txt, exactmatch, object, item_position, relative_to_word_in_x, relative_to_word_in_y, scroll=False):
    """
    Wait for a page element to appear.

    Args:
        txt (str): The text to wait for on the page.
        exactmatch (bool): Whether to perform an exact match for the element.
        object (str): The element object to interact with.
        item_position (int): The index of the item_position to interact with.
        relative_to_word_in_x (str): A reference word in x-axis direction to uniquely identify the element.
        relative_to_word_in_y (str): A reference word in y-axis direction to uniquely identify the element.
        scroll (bool): Whether to perform scrolling if the element is not found.

    Returns:
        tuple: The coordinates of the found element.
    """
    start_try = 0
    global found
    global new_file_path
    found = False
    exc = None
    stck = None

    while not found and start_try < retries:
        try:
            if not object == "":
                if not relative_to_word_in_x == "":
                    midpointR = _analyse_ocr_results(relative_to_word_in_x, exactmatch, )
                    midpoint = _find_image(object, item_position, midpointRY=midpointR[1])
                elif not relative_to_word_in_y == "":
                    midpointR = _analyse_ocr_results(relative_to_word_in_y, exactmatch, )
                    midpoint = _find_image(object, item_position, midpointRX=midpointR[0])
                else:
                    midpoint = _find_image(object)
                found = True
                break
            else:
                if not relative_to_word_in_x == "":
                    midpointR = _analyse_ocr_results(relative_to_word_in_x, exactmatch, item_position)
                    midpoint = _analyse_ocr_results(txt, exactmatch, item_position, midpointRY=midpointR[1])
                elif not relative_to_word_in_y == "":
                    top_leftR = _analyse_ocr_results(relative_to_word_in_y, exactmatch, item_position, y_axis=True)
                    midpoint = _analyse_ocr_results(txt, exactmatch, item_position, top_leftRX=top_leftR)
                else:
                    midpoint = _analyse_ocr_results(txt, exactmatch, item_position)
                found = True
                print(f"element '{txt}' found at location {midpoint} on the webpage")
                break
        except Exception as e:
            print(f"element not found...reading text from the webpage again")
            exc = e
        time.sleep(1)
        if not scroll:
            _take_screenshot()
        if object == "" and not scroll:
            _perform_ocr()
        elif object != "":
            if relative_to_word_in_y != "" or relative_to_word_in_x != "":
                _perform_ocr()
        if scroll:
            midpoint = False
            break
        start_try = start_try + 1
    if not found and not scroll:
        try:
            raise ElementNotFoundException(f"element '{txt}' not found within {retries} retries...exiting the script")
        except ElementNotFoundException as ce:
            print(f"Caught an exception: {ce.message}")
            report(f"Element '{txt}' not found", "Failed", "NA", screenshots_path)
            tear_down()
            exit(1)
    try:
        now = datetime.datetime.now()
        new_file_path = f"{screenshots_folder}/old_{screenshots_folder}/element_screenshot_{now.strftime('%Y-%m-%d-%H-%M-%S')}.png"
        shutil.copyfile(element_ss_path, new_file_path)
    except Exception as e:
        print(f"unable to copy analysed_screenshot to old_{screenshots_folder}.. getting exception {e}")

    return midpoint



import json
import os


class MyPackageConfig:
    def __init__(self):
        self.browser_path = "C:/Program Files/Google/Chrome/Application/chrome.exe"
        self.offsetx_default = 0
        self.offsety_default = 0
        self.mousespeed = 0.2
        self.screenshots_folder = 'screenshots'
        self.img_objects_folder = 'img_objects'
        self.retries = 5
        self.browser = 'chrome'
        self.options = '--start-maximized'
    def load_from_file(self, file_path):
        # Load configuration from a file (e.g., JSON)
        if os.path.exists(file_path):
            with open(file_path, 'r') as file:
                config_data = json.load(file)
                print('properties loaded from config.json file')
                self.browser_path = config_data.get('browser_path', self.browser_path)
                self.offsetx_default = config_data.get('offsetx_default', self.offsetx_default)
                self.offsety_default = config_data.get('offsety_default', self.offsety_default)
                self.mousespeed = config_data.get('mousespeed', self.mousespeed)
                self.screenshots_folder = config_data.get('screenshots_folder', self.screenshots_folder)
                self.img_objects_folder = config_data.get('img_objects_folder', self.img_objects_folder)
                self.retries = config_data.get('retries', self.retries)
                self.browser = config_data.get('browser', self.browser)
                self.options = config_data.get('options', self.options)
        self.screenshots_path = self.screenshots_folder + '/screenshot.png'
        self.output_path = self.screenshots_folder + '/identified_text_screenshot.png'
        self.element_ss_path = self.screenshots_folder + '/element_screenshot.png'

# Usage example
# Create an instance of the configuration class
config = MyPackageConfig()
config.load_from_file('config.json')



from datetime import datetime
from config_module import config
screenshots_folder = config.screenshots_folder
rows = ""

def generate_html():
        global rows
        # Get current date and time
        current_datetime = datetime.now()
        # Format date and time as dd-mmm-yyyy hh:mm:ss
        formatted_datetime = current_datetime.strftime("%d-%b-%Y %H:%M:%S")
        html_content = f"""<!DOCTYPE html>
        <html lang="en">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <title>HTML Report</title>
            <style>
                body {{
                    font-family: Arial, sans-serif;
                    margin: 20px;
                }}
        
                table {{
                    width: 70%;
                    border-collapse: collapse;
                    margin-bottom: 20px;
                }}
        
                th, td {{
                    border: 1px solid #ddd;
                    padding: 8px;
                    text-align: left;
                }}
        
                th {{
                    background-color: #f2f2f2;
                }}
        
                tr {{
                    height: 100px;
                }}

        
                img {{
                    max-width: 50%;
                    height: auto;
                    cursor: grab
                }}
                #imageModal {{
                    display: none;
                    position: fixed;
                    top: 0;
                    left: 0;
                    width: 100%;
                    height: 100%;
                    background-color: rgba(0, 0, 0, 0.7);
                    justify-content: center;
                    align-item_positions: center;
                }}

                #modalContent {{
                    text-align: center;
                }}
        
                #modalImage {{
                    max-width: 80%;
                    max-height: 80%;
                    border: 2px solid #fff;
                    border-radius: 5px;
                    box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
                }}
            </style>
            <script>
        function openImageModal(imageSrc) {{
            var modal = document.getElementById('imageModal');
            var modalImg = document.getElementById('modalImage');
            modalImg.src = imageSrc;
            modal.style.display = 'flex';
            }}

        function closeImageModal() {{
            document.getElementById('imageModal').style.display = 'none';
        }}
    </script>
        </head>
        <body>
        
        <h1>HTML Report</h1>
        <h3>Date: {formatted_datetime} Hrs</h3>        
        <table>
            <tr>
                <th width="50%">Step</th>
                <th width="10%">Result</th>
                <th width="10%">Time Taken</th>
                <th width="30%">Screenshot</th>
            </tr>
            {rows}
        </table>
          <div id="imageModal" onclick="closeImageModal()">
            <div id="modalContent">
                <img id="modalImage" src="" alt="Modal Image">
            </div>
        </div>
        </body>
        </html>
        """
        return html_content
def report(step, status, time_delta, screenshot):
    global rows
    if status == "Failed":
       color = "#F2938C"
    else:
       color = "#46C646"
    rows += f"""
        <tr>
        <td>{step}</td>
        <td style="color: {color}; font-weight: bold">{status}</td>
         <td>{time_delta}</td>
        <td><img src='../{screenshot}' alt="Screenshot" onclick="openImageModal('../{screenshot}')"></td>
        </tr>
        """
    with open(screenshots_folder + '/output.html', 'w') as file:
        file.write(generate_html())
    #print(f"rows {rows}")


class ElementNotFondException(Exception):
    pass
class CustomBaseException(Exception):
    def __init__(self, message=None):
        self.message = message
        super().__init__(message)

class ElementNotFoundException(CustomBaseException):
    """Exception raised when element not found."""


class InvalidScrollOptionException(CustomBaseException):
    def __init__(self, message, cause=None):
        super().__init__(message)
        self.__cause__ = cause



import os
import sys
import cv2
import numpy as np
import browser_utils as br
import pyautogui
from config_module import config
browser = config.browser
screenshots_path = config.screenshots_path
img_objects_folder = config.img_objects_folder
element_ss_path = config.element_ss_path

def _take_screenshot():
    """
    Captures a screenshot of the active window and saves it to a file.

    The function identifies the active window, captures a screenshot of that window, and saves it as an image.

    Args:
        None

    Returns:
        None
    """
    if sys.platform == "win32":
        try:
            active_window = br._activate_window()
        except Exception as e:
            print("unable to activate browser window.. trying again")
            try:
                active_window = br._activate_window()
            except Exception as e:
                print(f"unable to activate browser window..Caught exception {e}")
                exit(1)
        window_x, window_y, window_width, window_height = active_window.left, active_window.top, active_window.width, active_window.height

        screenshot = pyautogui.screenshot(region=(window_x, window_y, window_width, window_height))

        screenshot.save(screenshots_path)

    elif sys.platform == "linux":
        from wand.image import Image
        os.system(f"import -window root {screenshots_path}")
    else:
        print(f"Currently {sys.platform} platform is not supported")
        exit(1)
def _find_image(object, item_position=1, midpointRX=-1, midpointRY=-1):
    """
    Finds an image in the captured screenshot and returns its midpoint coordinates.

    The function searches for a specified image within the captured screenshot and returns the midpoint coordinates
    if a match is found.

    Args:
        object (str): The filename of the image to search for.
        item_position (int): Index of the match if there are multiple matches. Defaults to 0.
        midpointRX (int): Expected X-coordinate of the midpoint. Defaults to -1.
        midpointRY (int): Expected Y-coordinate of the midpoint. Defaults to -1.

    Returns:
        tuple: A tuple containing the X and Y coordinates of the midpoint if a match is found, or None if no match is found.
    """
    _take_screenshot()
    larger_image = cv2.imread(screenshots_path)
    template = cv2.imread(f'{img_objects_folder}/{object}')
    result = cv2.matchTemplate(larger_image, template, cv2.TM_CCORR_NORMED)
    # cv2.normalize(result, result, 0, 1, cv2.NORM_MINMAX)
    min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
    confidence_threshold = 0.999  # Adjust as needed
    loc = np.where(result >= confidence_threshold)
    i = 0
    # Draw rectangles around the matches
    for pt in zip(*loc[::-1]):  # pt marks the location of the match
        h, w = template.shape[:2]
        top_left = pt
        bottom_right = (pt[0] + w, pt[1] + h)
        midpointX = (top_left[0] + bottom_right[0]) // 2
        midpointY = (top_left[1] + bottom_right[1]) // 2
        if not midpointRX == -1:
            if abs(midpointRX - midpointX) < 5:
                midpoint = midpointX, midpointY
                break
        elif not midpointRY == -1:
            #print(f'midpoint diff of {midpointRY},{midpointY}=={abs(midpointRY - midpointY)}')
            if abs(midpointRY - midpointY) < 5:
                midpoint = midpointX, midpointY
                break
        elif item_position == i + 1:
            midpoint = midpointX, midpointY
            break
        i += 1
    cv2.rectangle(larger_image, top_left, bottom_right, (0, 0, 255), 2)
    cv2.imwrite(element_ss_path, larger_image)
    print(f"element '{object}' found at the location {midpointX},{midpointY} on the webpage with confidence: {max_val}")
    return midpoint



import setuptools

setuptools.setup(
	name="realswift",
	version="3.0.4",
	author="Jagadish Dabbiru",
	author_email="jagadish.dabbiru@gmail.com",
	packages=["realswift"],
	description="AI automation tool leverages a cognitive approach inspired by the human brain's intuitive understanding of web interfaces",
	long_description="This project performs user interactions with OpenCV2 by integrating the PyAutoGUI library alongside custom utility functions. The script offers a range of functionalities, including simulating user actions like typing, clicking, scrolling, and hovering. These actions are executed within a defined window or specific area, enhancing the automation of user interactions",
	long_description_content_type="text/markdown",
	url="https://github.com/jagadish165/realswift",
	license='BSD-2-Clause',
classifiers=[
        "Topic :: Software Development :: Testing",
        "Topic :: Internet :: WWW/HTTP :: Browsers",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
	python_requires='>=3.8',
	install_requires=[
		'easyocr==1.7.1',
		'numpy==1.26.4',
		'opencv_python_headless==4.8.1.78',
		'Pillow==10.2.0',
		'psutil==5.9.5',
		'PyAutoGUI==0.9.54',
		'PyGetWindow==0.0.9',
		'setuptools==65.5.0',
		'wand==0.6.13'
	]
)



import datetime
import cv2
import easyocr
import math

from PIL import Image, ImageDraw

from config_module import config
screenshots_path = config.screenshots_path
output_path = config.output_path
element_ss_path = config.element_ss_path

def _perform_ocr():
    global results, image, time_delta
    image = cv2.imread(screenshots_path)
    time_initial = datetime.datetime.now()
    reader = easyocr.Reader(
        ['en'],
        gpu=True
    )

    # Perform OCR on the image
    print('reading text from webpage...')
    results = reader.readtext(image, batch_size=64, slope_ths=0.5)
    time_delta = datetime.datetime.now() - time_initial
    time_delta = str(time_delta)[:-4]
    print(f"time took to read all the text from webpage- {time_delta}")

def _analyse_ocr_results(txtToSearch, exactmatch, item_position, top_leftRX=-1, midpointRY=-1, y_axis=False):
    #time_initial = datetime.datetime.now()
    listValues = []
    list_top = []
    for (bbox, text, prob) in results:
        (top_left, top_right, bottom_right, bottom_left) = bbox
        top_left = tuple(map(int, top_left))
        bottom_right = tuple(map(int, bottom_right))
        top_leftX = top_left[0]
        top_leftY = top_left[1]
        midpointX = (top_left[0] + bottom_right[0]) // 2
        midpointY = (top_left[1] + bottom_right[1]) // 2
        if exactmatch:
            if text.lower() == txtToSearch.lower():
                #print(f'{text} -- {midpointX},{midpointY} top_leftX --{top_leftX}')
                if not top_leftRX == -1:
                    #print(f'in == top_left diff of {top_leftRX},{top_leftX}=={abs(top_leftRX - top_leftX)}')
                    if abs(top_leftRX - top_leftX) <= 10:
                        midpoint = midpointX, midpointY
                        listValues.append(midpoint)
                        topleft = top_leftX, top_leftY
                        list_top.append(topleft)
                elif not midpointRY == -1:
                    #print(f'in == top_left diff of {midpointRY},{midpointY}=={abs(midpointRY - midpointY)}')
                    if abs(midpointRY - midpointY) <= 10:
                        midpoint = midpointX, midpointY
                        listValues.append(midpoint)
                        topleft = top_leftX, top_leftY
                        list_top.append(topleft)
                else:
                    midpoint = midpointX, midpointY
                    top_left_found = top_leftX
                    listValues.append(midpoint)
                    topleft = top_leftX, top_leftY
                    list_top.append(topleft)
        else:
            if txtToSearch.lower() in text.lower():
                #print(f'{text} -- {midpointX},{midpointY} top_leftX --{top_leftX}')
                if not top_leftRX == -1:
                    #print(f'in == top_left diff of {top_leftRX},{top_leftX}=={abs(top_leftRX - top_leftX)}')
                    if abs(top_leftRX - top_leftX) < 5:
                        midpoint = midpointX, midpointY
                        listValues.append(midpoint)
                        topleft = top_leftX, top_leftY
                        list_top.append(topleft)
                elif not midpointRY == -1:
                    #print(f'in == top_left diff of {midpointRY},{midpointY}=={abs(midpointRY - midpointY)}')
                    if abs(midpointRY - midpointY) < 5:
                        midpoint = midpointX, midpointY
                        listValues.append(midpoint)
                        topleft = top_leftX, top_leftY
                        list_top.append(topleft)
                else:
                    midpoint = midpointX, midpointY
                    top_left_found = top_leftX
                    listValues.append(midpoint)
                    topleft = top_leftX, top_leftY
                    list_top.append(topleft)
                    # Draw a bounding box around the detected text
        cv2.rectangle(image, top_left, bottom_right, (0, 255, 0), 2)
        # print(f'Bounding Box Coordinates:{text}, {top_left}, {bottom_right}')

    cv2.imwrite(output_path, image)
    distances = [math.sqrt(x ** 2 + y ** 2) for x, y in listValues]
    distances_tl = [math.sqrt(a ** 2 + b ** 2) for a, b in list_top]

    # Create a list of (distance, coordinate) pairs
    distance_coordinate_pairs = list(zip(distances, listValues))
    distance_coordinate_pairs_tl = list(zip(distances_tl, list_top))
    # Sort the pairs by distance in ascending order
    sorted_pairs = sorted(distance_coordinate_pairs, key=lambda x: x[0])
    sorted_pairs_tl = sorted(distance_coordinate_pairs_tl, key=lambda a: a[0])

    # Extract the sorted coordinates
    sorted_coordinates = [pair[1] for pair in sorted_pairs]
    sorted_coordinates_tl = [pair[1] for pair in sorted_pairs_tl]
    top_left_x, top_left_y = sorted_coordinates_tl[item_position-1]
    midpoint_x, midpoint_y = sorted_coordinates[item_position-1]
    bottom_right_x = 2 * midpoint_x - top_left_x
    bottom_right_y = 2 * midpoint_y - top_left_y
    top_left = top_left_x, top_left_y
    bottom_right = bottom_right_x, bottom_right_y
    coordinates = top_left_x, top_left_y, bottom_right_x, bottom_right_y
    #time_delta = datetime.datetime.now() - time_initial
    #print(f"Time took to analyse - {time_delta}")
    #cv2.rectangle(image, top_left, bottom_right, (255, 0, 0), 2)
    #cv2.imwrite(output_path, image)
    ss = Image.open(screenshots_path)
    draw = ImageDraw.Draw(ss)
    draw.rectangle(coordinates, outline="red", width=3)
    ss.save(element_ss_path)


    if y_axis:
        return top_left_found
    else:
        return sorted_coordinates[item_position-1]
